#
# Copyright (c) 2021 Software AG, Darmstadt, Germany and/or its licensors
#
# SPDX-License-Identifier: Apache-2.0
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""c8ylp version"""
import os
import importlib

__ROOT_DIR__ = os.path.dirname(os.path.abspath(__file__))  # This is your Project Root


def _get_version():
    # pylint: disable=bare-except, import-outside-toplevel
    try:
        # Needs to be dynamically imported as the version file is not
        # committed to the project as it is generated during the build.
        version = importlib.import_module("c8ylp._version")
        return version.version
    except:
        return "0.0.1"


__version__ = _get_version()
